﻿using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Polly;
using Polly.Caching;
using Polly.Registry;

namespace CachePolicyOutsideHttpClientFactory.Controllers
{
    [Produces("application/json")]
    [Route("api/[controller]")]
    public class StockMgtController : Controller
    {
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly IPolicyRegistry<string> _policyRegistry;

        public StockMgtController(IHttpClientFactory httpClientFactory, IPolicyRegistry<string> policyRegistry)
        {
            _httpClientFactory = httpClientFactory;
            _policyRegistry = policyRegistry;
        }

        [HttpGet("{id}")]
        public async Task<IActionResult> Get(int id)
        {
            string requestEndpoint = $"inventory/{id}";
            var httpClient = _httpClientFactory.CreateClient("RemoteServer");

            var cachePolicy = _policyRegistry.Get<CachePolicy<HttpResponseMessage>>("CachingPolicy");
            Context policyExecutionContext = new Context($"GetInventoryById-{id}");

            HttpResponseMessage response = await cachePolicy.ExecuteAsync(
                context => httpClient.GetAsync(requestEndpoint), policyExecutionContext);

            if (response.IsSuccessStatusCode)
            {
                int itemsInStock = JsonConvert.DeserializeObject<int>(await response.Content.ReadAsStringAsync());
                return Ok(itemsInStock);
            }

            return StatusCode((int)response.StatusCode, response.Content.ReadAsStringAsync());
        }
    }
}
